//**************************************************************
//                      checksum.c
//                        V0.1.1
//                     25 October 2000
//
// - David Haywood 30AUG99
//
// - Michael Pearce - Modified For PIC12C6xx Processor use 28 August 2000
//    For use with HiTech - PIC C.
//
//**************************************************************
// V0.1.1 -25 October 2000
// Customised the Function names for the Heater2 Project
//**************************************************************
// V0.1.0 - 28 August 2000
// Mods done by Mike Pearce to make it work on a PIC12C67x
//**************************************************************


// MIKE,
//
// The CHECKSUM function calculates the CRC for incoming bytes stored in the
// array BYTE.  The returned CRC value can then be compared with the value
// given by the device.
//
// The other parameter required is the size (length) of the array BYTE.  This
// can be calculated using the "sizeof" function when CHECKSUM is called.
// eg. CHECKSUM(BYTE, sizeof(BYTE))
//
// The read_temp_sensor function (listed below CHECKSUM) gives an example of
// how CHECKSUM can be used to check the integrity of a reading from a DS1820
//
// - Originally from DALLAS SEMICONDUCTORS Application Note 27
//  Understanding & Using Cyclic Redundancy Checks with Dallas iButton Products
//
// - David Haywood 30AUG99
//
// - Michael Pearce - Modified For PIC12C6xx Processor use 28 August 2000
//    For use with HiTech - PIC C.
//

 // DECLARE AND SET VARIABLES
const unsigned char BYTE_ARRAY [256] =
 {
  0, 94, 188, 226, 97, 63, 221, 131, 194, 156, 126, 32, 163, 253, 31, 65,
  157, 195, 33, 127, 252, 162, 64, 30, 95, 1, 227, 189, 62, 96, 130, 220,
  35, 125, 159, 193, 66, 28, 254, 160, 225, 191, 93, 3, 128, 222, 60, 98,
  190, 224, 2, 92, 223, 129, 99, 61, 124, 34, 192, 158, 29, 67, 161, 255,
  70, 24, 250, 164, 39, 121, 155, 197, 132, 218, 56, 102, 229, 187, 89, 7,
  219, 133, 103, 57, 186, 228, 6, 88, 25, 71, 165, 251, 120, 38, 196, 154,
  101, 59, 217, 135, 4, 90, 184, 230, 167, 249, 27, 69, 198, 152, 122, 36,
  248, 166, 68, 26, 153, 199, 37, 123, 58, 100, 134, 216, 91, 5, 231, 185,
  140, 210, 48, 110, 237, 179, 81, 15, 78, 16, 242, 172, 47, 113, 147, 205,
  17, 79, 173, 243, 112, 46, 204, 146, 211, 141, 111, 49, 178, 236, 14, 80,
  175, 241, 19, 77, 206, 144, 114, 44, 109, 51, 209, 143, 12, 82, 176, 238,
  50, 108, 142, 208, 83, 13, 239, 177, 240, 174, 76, 18, 145, 207, 45, 115,
  202, 148, 118, 40, 171, 245, 23, 73, 8, 86, 180, 234, 105, 55, 213, 139,
  87, 9, 235, 181, 54, 104, 138, 212, 149, 203, 41, 119, 244, 170, 72, 22,
  233, 183, 85, 11, 136, 214, 52, 106, 43, 117, 151, 201, 74, 20, 246, 168,
  116, 42, 200, 150, 21, 75, 169, 247, 182, 232, 10, 84, 215, 137, 107, 53
 };
 // lookup array for CHECKSUM calcs


// Calculates DOW CRC value for a stream of bytes
// unsigned char CHECKSUM (unsigned char *BYTE, int size_of_BYTE)
unsigned char PROBE_CHECKSUM (unsigned char *BYTE, int size_of_BYTE)
{
 // DECLARE VARIABLES
 int BYTE_loop;
 unsigned char CRC;

 // FUNCTION BODY;
 CRC = 0;
 for (BYTE_loop = 0; BYTE_loop < size_of_BYTE; BYTE_loop++)
 {
  CRC = BYTE_ARRAY[CRC ^ BYTE[BYTE_loop]]; // look up array for appropriate calc
 }
 return (CRC);
}

